"""Weather Forecast"""

from datetime import date, datetime
from typing import Literal, Mapping, Optional, Sequence, Tuple, Union

from ..types import (
    CellSelection,
    Coordinate,
    Number,
    PrecipitationUnit,
    TemperatureUnit,
    TimeFormat,
    WindSpeedUnit
)
from ..utils import (
    ensure_sequence,
    optional_date_to_param,
    optional_datetime_to_param,
    optional_int_to_param,
    optional_number_to_param,
    optional_string_array_to_param,
    remove_none_from_dict,
)

Hourly = Literal[
    'temperature_2m',
    'relative_humidity_2m',
    'dew_point_2m',
    'apparent_temperature',
    'pressure_msl',
    'surface_pressure',
    'cloud_cover',
    'cloud_cover_low',
    'cloud_cover_mid',
    'cloud_cover_high',
    'wind_speed_10m',
    'wind_speed_80m',
    'wind_speed_120m',
    'wind_speed_180m',
    'wind_direction_10m',
    'wind_direction_80m',
    'wind_direction_120m',
    'wind_direction_180m',
    'wind_gusts_10m',
    'shortwave_radiation',
    'direct_radiation',
    'direct_normal_irradiance',
    'diffuse_radiation',
    'global_tilted_irradiance',
    'vapour_pressure_deficit',
    'cape',
    'evapotranspiration',
    'et0_fao_evapotranspiration',
    'precipitation',
    'snowfall',
    'precipitation_probability',
    'rain',
    'showers',
    'weather_code',
    'snow_depth',
    'freezing_level_height',
    'visibility',
    'soil_temperature_0cm',
    'soil_temperature_6cm',
    'soil_temperature_18cm',
    'soil_temperature_54cm',
    'soil_moisture_0_to_1cm',
    'soil_moisture_1_to_3cm',
    'soil_moisture_3_to_9cm',
    'soil_moisture_9_to_27cm',
    'soil_moisture_27_to_81cm',
    'is_day',

    # Additional
    'uv_index',
    'uv_index_clear_sky',
    'is_day',
    'cape',
    'freezing_level_height',
    'sunshine_duration',

    # Solar radiation
    'shortwave_radiation',
    'direct_radiation',
    'diffuse_radiation',
    'direct_normal_irradiance',
    'terrestrial_radiation',
    'shortwave_radiation_instant',
    'direct_radiation_instant',
    'diffuse_radiation_instant',
    'direct_normal_irradiance_instant',
    'terrestrial_radiation_instant',
]

Daily = Literal[
    'temperature_2m_max',
    'temperature_2m_min',
    'apparent_temperature_max',
    'apparent_temperature_min',
    'precipitation_sum',
    'rain_sum',
    'showers_sum',
    'snowfall_sum',
    'precipitation_hours',
    'precipitation_probability_max',
    'precipitation_probability_min',
    'precipitation_probability_mean',
    'weather_code',
    'sunrise',
    'sunset',
    'sunshine_duration',
    'daylight_duration',
    'wind_speed_10m_max',
    'wind_gusts_10m_max',
    'wind_direction_10m_dominant',
    'shortwave_radiation_sum',
    'et0_fao_evapotranspiration',
    'uv_index_max',
    'uv_index_clear_sky_max'
]

Current = Literal[
    'temperature_2m',
    'relative_humidity_2m',
    'apparent_temperature',
    'is_day',
    'precipitation',
    'rain',
    'showers',
    'snowfall',
    'weather_code',
    'cloud_cover',
    'pressure_msl',
    'surface_pressure',
    'wind_speed_10m',
    'wind_direction_10m',
    'wind_gusts_10m'
]

WeatherModel = Literal[
    'best_match',
    'ecmwf_ifs04',
    'cma_grapes_global',
    'bom_access_global',
    'metno_nordic',
    'gfs_seamless',
    'gfs_global',
    'gfs_hrrr',
    'jma_seamless',
    'jma_msm',
    'jma_gsm',
    'icon_seamless',
    'icon_global',
    'icon_eu',
    'icon_d2',
    'gem_seamless',
    'gem_global',
    'gem_regional',
    'gem_hrdps_continental',
    'meteofrance_seamless',
    'meteofrance_arpege_world',
    'meteofrance_arpege_europe',
    'meteofrance_arome_france',
    'meteofrance_arome_france_hd'
]


def prepare_weather_forecast_request(
        coordinate: Union[Coordinate, Sequence[Coordinate]],
        elevation: Optional[Number] = None,
        hourly: Optional[Sequence[Hourly]] = None,
        daily: Optional[Sequence[Daily]] = None,
        current: Optional[Sequence[Current]] = None,
        temperature_unit: Optional[TemperatureUnit] = None,
        wind_speed_unit: Optional[WindSpeedUnit] = None,
        precipitation_unit: Optional[PrecipitationUnit] = None,
        timeformat: Optional[TimeFormat] = None,
        timezone: Optional[str] = None,
        past_days: Optional[int] = None,
        forecast_days: Optional[int] = None,
        forecast_hours: Optional[int] = None,
        forecast_minutely_15: Optional[int] = None,
        past_hours: Optional[int] = None,
        past_minutely_15: Optional[int] = None,
        start_date: Optional[date] = None,
        end_date: Optional[date] = None,
        start_hour: Optional[datetime] = None,
        end_hour: Optional[datetime] = None,
        start_minutely_15: Optional[datetime] = None,
        end_minutely_15: Optional[datetime] = None,
        models: Optional[Sequence[WeatherModel]] = None,
        cell_selection: Optional[CellSelection] = None,
        apikey: Optional[str] = None
) -> Tuple[str, Mapping[str, str]]:
    url = "https://api.open-meteo.com/v1/forecast"

    coordinate = ensure_sequence(coordinate)

    params = remove_none_from_dict({
        'latitude': ','.join([str(latitude) for latitude, _ in coordinate]),
        'longitude': ','.join([str(longitude) for _, longitude in coordinate]),
        'elevation': optional_number_to_param(elevation),
        'hourly': optional_string_array_to_param(hourly),
        'daily': optional_string_array_to_param(daily),
        'current': optional_string_array_to_param(current),
        'temperature_unit': temperature_unit,
        'wind_speed_unit': wind_speed_unit,
        'precipitation_unit': precipitation_unit,
        'timeformat': timeformat,
        'timezone': timezone,
        'past_days': optional_int_to_param(past_days, 0, 92),
        'forecast_days': optional_int_to_param(forecast_days, 0, 16),
        'forecast_hours': optional_int_to_param(forecast_hours, 0),
        'forecast_minutely_15': optional_int_to_param(forecast_minutely_15, 0),
        'past_hours': optional_int_to_param(past_hours, 0),
        'past_minutely_15': optional_int_to_param(past_minutely_15, 0),
        'start_date': optional_date_to_param(start_date),
        'end_date': optional_date_to_param(end_date),
        'start_hour': optional_datetime_to_param(start_hour),
        'end_hour': optional_datetime_to_param(end_hour),
        'start_minutely_15': optional_datetime_to_param(start_minutely_15),
        'end_minutely_15': optional_datetime_to_param(end_minutely_15),
        'models': optional_string_array_to_param(models),
        'cell_selection': cell_selection,
        'apikey': apikey
    })

    return url, params
